@@ -32,6 +32,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "../../src/client/registry.h"
 #include "../../src/client/seat.h"
 #include "../../src/client/shm_pool.h"
+#include "../../src/client/touch.h"
 #include "../../src/server/buffer_interface.h"
 #include "../../src/server/compositor_interface.h"
 #include "../../src/server/datadevicemanager_interface.h"
@@ -60,10 +61,12 @@ private Q_SLOTS:
     void testPointer();
     void testPointerButton_data();
     void testPointerButton();
+    void testCursor();
     void testKeyboard();
     void testCast();
     void testDestroy();
     void testSelection();
+    void testTouch();
     // TODO: add test for keymap
 
 private:
@@ -73,6 +76,7 @@ private:
     KWayland::Client::ConnectionThread *m_connection;
     KWayland::Client::Compositor *m_compositor;
     KWayland::Client::Seat *m_seat;
+    KWayland::Client::ShmPool *m_shm;
     KWayland::Client::EventQueue *m_queue;
     QThread *m_thread;
 };
@@ -87,6 +91,7 @@ TestWaylandSeat::TestWaylandSeat(QObject *parent)
     , m_connection(nullptr)
     , m_compositor(nullptr)
     , m_seat(nullptr)
+    , m_shm(nullptr)
     , m_queue(nullptr)
     , m_thread(nullptr)
 {
@@ -100,6 +105,7 @@ void TestWaylandSeat::init()
     m_display->setSocketName(s_socketName);
     m_display->start();
     QVERIFY(m_display->isRunning());
+    m_display->createShm();
 
     m_compositorInterface = m_display->createCompositor(m_display);
     QVERIFY(m_compositorInterface);
@@ -132,6 +138,7 @@ void TestWaylandSeat::init()
     KWayland::Client::Registry registry;
     QSignalSpy compositorSpy(&registry, SIGNAL(compositorAnnounced(quint32,quint32)));
     QSignalSpy seatSpy(&registry, SIGNAL(seatAnnounced(quint32,quint32)));
+    QSignalSpy shmSpy(&registry, SIGNAL(shmAnnounced(quint32,quint32)));
     registry.setEventQueue(m_queue);
     registry.create(m_connection->display());
     QVERIFY(registry.isValid());
@@ -152,10 +159,18 @@ void TestWaylandSeat::init()
     m_seat = registry.createSeat(seatSpy.first().first().value<quint32>(), seatSpy.first().last().value<quint32>(), this);
     QSignalSpy nameSpy(m_seat, SIGNAL(nameChanged(QString)));
     QVERIFY(nameSpy.wait());
+
+    m_shm = new KWayland::Client::ShmPool(this);
+    m_shm->setup(registry.bindShm(shmSpy.first().first().value<quint32>(), shmSpy.first().last().value<quint32>()));
+    QVERIFY(m_shm->isValid());
 }
 
 void TestWaylandSeat::cleanup()
 {
+    if (m_shm) {
+        delete m_shm;
+        m_shm = nullptr;
+    }
     if (m_seat) {
         delete m_seat;
         m_seat = nullptr;
@@ -514,6 +529,103 @@ void TestWaylandSeat::testPointerButton()
     QCOMPARE(buttonChangedSpy.last().at(3).value<KWayland::Client::Pointer::ButtonState>(), Pointer::ButtonState::Released);
 }
 
+void TestWaylandSeat::testCursor()
+{
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+
+    QSignalSpy pointerSpy(m_seat, SIGNAL(hasPointerChanged(bool)));
+    QVERIFY(pointerSpy.isValid());
+    m_seatInterface->setHasPointer(true);
+    QVERIFY(pointerSpy.wait());
+
+    QSignalSpy surfaceCreatedSpy(m_compositorInterface, SIGNAL(surfaceCreated(KWayland::Server::SurfaceInterface*)));
+    QVERIFY(surfaceCreatedSpy.isValid());
+    m_compositor->createSurface(m_compositor);
+    QVERIFY(surfaceCreatedSpy.wait());
+    SurfaceInterface *serverSurface = surfaceCreatedSpy.first().first().value<KWayland::Server::SurfaceInterface*>();
+    QVERIFY(serverSurface);
+
+    QScopedPointer<Pointer> p(m_seat->createPointer());
+    QVERIFY(p->isValid());
+    wl_display_flush(m_connection->display());
+    QCoreApplication::processEvents();
+
+    QSignalSpy enteredSpy(p.data(), SIGNAL(entered(quint32,QPointF)));
+    QVERIFY(enteredSpy.isValid());
+
+    m_seatInterface->setPointerPos(QPoint(20, 18));
+    m_seatInterface->setFocusedPointerSurface(serverSurface, QPoint(10, 15));
+    quint32 serial = m_seatInterface->display()->serial();
+    QVERIFY(enteredSpy.wait());
+    QCOMPARE(enteredSpy.first().first().value<quint32>(), serial);
+    QVERIFY(m_seatInterface->focusedPointerSurface());
+    QVERIFY(m_seatInterface->focusedPointer());
+    QVERIFY(!m_seatInterface->focusedPointer()->cursor());
+
+    QSignalSpy cursorChangedSpy(m_seatInterface->focusedPointer(), SIGNAL(cursorChanged()));
+    QVERIFY(cursorChangedSpy.isValid());
+    // just remove the pointer
+    p->setCursor(nullptr);
+    QVERIFY(cursorChangedSpy.wait());
+    QCOMPARE(cursorChangedSpy.count(), 1);
+    auto cursor = m_seatInterface->focusedPointer()->cursor();
+    QVERIFY(cursor);
+    QVERIFY(!cursor->surface());
+    QCOMPARE(cursor->hotspot(), QPoint());
+    QCOMPARE(cursor->enteredSerial(), serial);
+
+    QSignalSpy hotspotChangedSpy(cursor, SIGNAL(hotspotChanged()));
+    QVERIFY(hotspotChangedSpy.isValid());
+    QSignalSpy surfaceChangedSpy(cursor, SIGNAL(surfaceChanged()));
+    QVERIFY(surfaceChangedSpy.isValid());
+    QSignalSpy enteredSerialChangedSpy(cursor, SIGNAL(enteredSerialChanged()));
+    QVERIFY(enteredSerialChangedSpy.isValid());
+    QSignalSpy changedSpy(cursor, SIGNAL(changed()));
+    QVERIFY(changedSpy.isValid());
+
+    // test changing hotspot
+    p->setCursor(nullptr, QPoint(1, 2));
+    QVERIFY(hotspotChangedSpy.wait());
+    QCOMPARE(hotspotChangedSpy.count(), 1);
+    QCOMPARE(changedSpy.count(), 1);
+    QCOMPARE(cursorChangedSpy.count(), 2);
+    QCOMPARE(cursor->hotspot(), QPoint(1, 2));
+    QVERIFY(enteredSerialChangedSpy.isEmpty());
+    QVERIFY(surfaceChangedSpy.isEmpty());
+
+    // set surface
+    auto cursorSurface = m_compositor->createSurface(m_compositor);
+    QVERIFY(cursorSurface->isValid());
+    p->setCursor(cursorSurface, QPoint(1, 2));
+    QVERIFY(surfaceChangedSpy.wait());
+    QCOMPARE(surfaceChangedSpy.count(), 1);
+    QCOMPARE(changedSpy.count(), 2);
+    QCOMPARE(cursorChangedSpy.count(), 3);
+    QVERIFY(enteredSerialChangedSpy.isEmpty());
+    QCOMPARE(cursor->hotspot(), QPoint(1, 2));
+    QVERIFY(cursor->surface());
+
+    // and add an image to the surface
+    QImage img(QSize(10, 20), QImage::Format_RGB32);
+    img.fill(Qt::red);
+    cursorSurface->attachBuffer(m_shm->createBuffer(img));
+    cursorSurface->damage(QRect(0, 0, 10, 20));
+    cursorSurface->commit(Surface::CommitFlag::None);
+    QVERIFY(changedSpy.wait());
+    QCOMPARE(changedSpy.count(), 3);
+    QCOMPARE(cursorChangedSpy.count(), 4);
+    QCOMPARE(surfaceChangedSpy.count(), 1);
+    QCOMPARE(cursor->surface()->buffer()->data(), img);
+
+    p->hideCursor();
+    QVERIFY(surfaceChangedSpy.wait());
+    QCOMPARE(changedSpy.count(), 4);
+    QCOMPARE(cursorChangedSpy.count(), 5);
+    QCOMPARE(surfaceChangedSpy.count(), 2);
+    QVERIFY(!cursor->surface());
+}
+
 void TestWaylandSeat::testKeyboard()
 {
     using namespace KWayland::Client;
@@ -682,9 +794,17 @@ void TestWaylandSeat::testDestroy()
     Pointer *p = m_seat->createPointer(m_seat);
     QVERIFY(p->isValid());
 
+    QSignalSpy touchSpy(m_seat, SIGNAL(hasTouchChanged(bool)));
+    QVERIFY(touchSpy.isValid());
+    m_seatInterface->setHasTouch(true);
+    QVERIFY(touchSpy.wait());
+    Touch *t = m_seat->createTouch(m_seat);
+    QVERIFY(t->isValid());
+
     delete m_compositor;
     m_compositor = nullptr;
     connect(m_connection, &ConnectionThread::connectionDied, m_seat, &Seat::destroy);
+    connect(m_connection, &ConnectionThread::connectionDied, m_shm, &ShmPool::destroy);
     connect(m_connection, &ConnectionThread::connectionDied, m_queue, &EventQueue::destroy);
     QVERIFY(m_seat->isValid());
 
@@ -700,11 +820,13 @@ void TestWaylandSeat::testDestroy()
     QVERIFY(!m_seat->isValid());
     QVERIFY(!k->isValid());
     QVERIFY(!p->isValid());
+    QVERIFY(!t->isValid());
 
     // calling destroy again should not fail
     m_seat->destroy();
     k->destroy();
     p->destroy();
+    t->destroy();
 }
 
 void TestWaylandSeat::testSelection()
@@ -782,5 +904,194 @@ void TestWaylandSeat::testSelection()
     QCOMPARE(selectionSpy.count(), 1);
 }
 
+void TestWaylandSeat::testTouch()
+{
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+
+    QSignalSpy touchSpy(m_seat, SIGNAL(hasTouchChanged(bool)));
+    QVERIFY(touchSpy.isValid());
+    m_seatInterface->setHasTouch(true);
+    QVERIFY(touchSpy.wait());
+
+    // create the surface
+    QSignalSpy surfaceCreatedSpy(m_compositorInterface, SIGNAL(surfaceCreated(KWayland::Server::SurfaceInterface*)));
+    QVERIFY(surfaceCreatedSpy.isValid());
+    Surface *s = m_compositor->createSurface(m_compositor);
+    QVERIFY(surfaceCreatedSpy.wait());
+    SurfaceInterface *serverSurface = surfaceCreatedSpy.first().first().value<KWayland::Server::SurfaceInterface*>();
+    QVERIFY(serverSurface);
+
+    m_seatInterface->setFocusedTouchSurface(serverSurface);
+    // no keyboard yet
+    QCOMPARE(m_seatInterface->focusedTouchSurface(), serverSurface);
+    QVERIFY(!m_seatInterface->focusedTouch());
+
+    QSignalSpy touchCreatedSpy(m_seatInterface, SIGNAL(touchCreated(KWayland::Server::TouchInterface*)));
+    QVERIFY(touchCreatedSpy.isValid());
+    Touch *touch = m_seat->createTouch(m_seat);
+    QVERIFY(touch->isValid());
+    QVERIFY(touchCreatedSpy.wait());
+    QVERIFY(m_seatInterface->focusedTouch());
+    QCOMPARE(touchCreatedSpy.first().first().value<KWayland::Server::TouchInterface*>(), m_seatInterface->focusedTouch());
+
+    QSignalSpy sequenceStartedSpy(touch, SIGNAL(sequenceStarted(KWayland::Client::TouchPoint*)));
+    QVERIFY(sequenceStartedSpy.isValid());
+    QSignalSpy sequenceEndedSpy(touch, SIGNAL(sequenceEnded()));
+    QVERIFY(sequenceEndedSpy.isValid());
+    QSignalSpy sequenceCanceledSpy(touch, SIGNAL(sequenceCanceled()));
+    QVERIFY(sequenceCanceledSpy.isValid());
+    QSignalSpy frameEndedSpy(touch, SIGNAL(frameEnded()));
+    QVERIFY(frameEndedSpy.isValid());
+    QSignalSpy pointAddedSpy(touch, SIGNAL(pointAdded(KWayland::Client::TouchPoint*)));
+    QVERIFY(pointAddedSpy.isValid());
+    QSignalSpy pointMovedSpy(touch, SIGNAL(pointMoved(KWayland::Client::TouchPoint*)));
+    QVERIFY(pointMovedSpy.isValid());
+    QSignalSpy pointRemovedSpy(touch, SIGNAL(pointRemoved(KWayland::Client::TouchPoint*)));
+    QVERIFY(pointRemovedSpy.isValid());
+
+    // try a few things
+    m_seatInterface->setFocusedTouchSurfacePosition(QPointF(10, 20));
+    QCOMPARE(m_seatInterface->focusedTouchSurfacePosition(), QPointF(10, 20));
+    m_seatInterface->setTimestamp(1);
+    QCOMPARE(m_seatInterface->touchDown(QPointF(15, 26)), 0);
+    QVERIFY(sequenceStartedSpy.wait());
+    QCOMPARE(sequenceStartedSpy.count(), 1);
+    QCOMPARE(sequenceEndedSpy.count(), 0);
+    QCOMPARE(sequenceCanceledSpy.count(), 0);
+    QCOMPARE(frameEndedSpy.count(), 0);
+    QCOMPARE(pointAddedSpy.count(), 0);
+    QCOMPARE(pointMovedSpy.count(), 0);
+    QCOMPARE(pointRemovedSpy.count(), 0);
+    TouchPoint *tp = sequenceStartedSpy.first().first().value<TouchPoint*>();
+    QVERIFY(tp);
+    QCOMPARE(tp->downSerial(), m_seatInterface->display()->serial());
+    QCOMPARE(tp->id(), 0);
+    QVERIFY(tp->isDown());
+    QCOMPARE(tp->position(), QPointF(5, 6));
+    QCOMPARE(tp->positions().size(), 1);
+    QCOMPARE(tp->time(), 1u);
+    QCOMPARE(tp->timestamps().count(), 1);
+    QCOMPARE(tp->upSerial(), 0u);
+    QCOMPARE(tp->surface().data(), s);
+    QCOMPARE(touch->sequence().count(), 1);
+    QCOMPARE(touch->sequence().first(), tp);
+
+    // let's end the frame
+    m_seatInterface->touchFrame();
+    QVERIFY(frameEndedSpy.wait());
+    QCOMPARE(frameEndedSpy.count(), 1);
+
+    // move the one point
+    m_seatInterface->setTimestamp(2);
+    m_seatInterface->touchMove(0, QPointF(10, 20));
+    m_seatInterface->touchFrame();
+    QVERIFY(frameEndedSpy.wait());
+    QCOMPARE(sequenceStartedSpy.count(), 1);
+    QCOMPARE(sequenceEndedSpy.count(), 0);
+    QCOMPARE(sequenceCanceledSpy.count(), 0);
+    QCOMPARE(frameEndedSpy.count(), 2);
+    QCOMPARE(pointAddedSpy.count(), 0);
+    QCOMPARE(pointMovedSpy.count(), 1);
+    QCOMPARE(pointRemovedSpy.count(), 0);
+    QCOMPARE(pointMovedSpy.first().first().value<TouchPoint*>(), tp);
+
+    QCOMPARE(tp->id(), 0);
+    QVERIFY(tp->isDown());
+    QCOMPARE(tp->position(), QPointF(0, 0));
+    QCOMPARE(tp->positions().size(), 2);
+    QCOMPARE(tp->time(), 2u);
+    QCOMPARE(tp->timestamps().count(), 2);
+    QCOMPARE(tp->upSerial(), 0u);
+    QCOMPARE(tp->surface().data(), s);
+
+    // add onther point
+    m_seatInterface->setTimestamp(3);
+    QCOMPARE(m_seatInterface->touchDown(QPointF(15, 26)), 1);
+    m_seatInterface->touchFrame();
+    QVERIFY(frameEndedSpy.wait());
+    QCOMPARE(sequenceStartedSpy.count(), 1);
+    QCOMPARE(sequenceEndedSpy.count(), 0);
+    QCOMPARE(sequenceCanceledSpy.count(), 0);
+    QCOMPARE(frameEndedSpy.count(), 3);
+    QCOMPARE(pointAddedSpy.count(), 1);
+    QCOMPARE(pointMovedSpy.count(), 1);
+    QCOMPARE(pointRemovedSpy.count(), 0);
+    QCOMPARE(touch->sequence().count(), 2);
+    QCOMPARE(touch->sequence().first(), tp);
+    TouchPoint *tp2 = pointAddedSpy.first().first().value<TouchPoint*>();
+    QVERIFY(tp2);
+    QCOMPARE(touch->sequence().last(), tp2);
+    QCOMPARE(tp2->id(), 1);
+    QVERIFY(tp2->isDown());
+    QCOMPARE(tp2->position(), QPointF(5, 6));
+    QCOMPARE(tp2->positions().size(), 1);
+    QCOMPARE(tp2->time(), 3u);
+    QCOMPARE(tp2->timestamps().count(), 1);
+    QCOMPARE(tp2->upSerial(), 0u);
+    QCOMPARE(tp2->surface().data(), s);
+
+    // send it an up
+    m_seatInterface->setTimestamp(4);
+    m_seatInterface->touchUp(1);
+    m_seatInterface->touchFrame();
+    QVERIFY(frameEndedSpy.wait());
+    QCOMPARE(sequenceStartedSpy.count(), 1);
+    QCOMPARE(sequenceEndedSpy.count(), 0);
+    QCOMPARE(sequenceCanceledSpy.count(), 0);
+    QCOMPARE(frameEndedSpy.count(), 4);
+    QCOMPARE(pointAddedSpy.count(), 1);
+    QCOMPARE(pointMovedSpy.count(), 1);
+    QCOMPARE(pointRemovedSpy.count(), 1);
+    QCOMPARE(pointRemovedSpy.first().first().value<TouchPoint*>(), tp2);
+    QCOMPARE(tp2->id(), 1);
+    QVERIFY(!tp2->isDown());
+    QCOMPARE(tp2->position(), QPointF(5, 6));
+    QCOMPARE(tp2->positions().size(), 1);
+    QCOMPARE(tp2->time(), 4u);
+    QCOMPARE(tp2->timestamps().count(), 2);
+    QCOMPARE(tp2->upSerial(), m_seatInterface->display()->serial());
+    QCOMPARE(tp2->surface().data(), s);
+
+    // send another down and up
+    m_seatInterface->setTimestamp(5);
+    QCOMPARE(m_seatInterface->touchDown(QPointF(15, 26)), 1);
+    m_seatInterface->touchFrame();
+    m_seatInterface->setTimestamp(6);
+    m_seatInterface->touchUp(1);
+    // and send an up for the first point
+    m_seatInterface->touchUp(0);
+    m_seatInterface->touchFrame();
+    QVERIFY(frameEndedSpy.wait());
+    QCOMPARE(sequenceStartedSpy.count(), 1);
+    QCOMPARE(sequenceEndedSpy.count(), 1);
+    QCOMPARE(sequenceCanceledSpy.count(), 0);
+    QCOMPARE(frameEndedSpy.count(), 6);
+    QCOMPARE(pointAddedSpy.count(), 2);
+    QCOMPARE(pointMovedSpy.count(), 1);
+    QCOMPARE(pointRemovedSpy.count(), 3);
+    QCOMPARE(touch->sequence().count(), 3);
+    QVERIFY(!touch->sequence().at(0)->isDown());
+    QVERIFY(!touch->sequence().at(1)->isDown());
+    QVERIFY(!touch->sequence().at(2)->isDown());
+    QVERIFY(!m_seatInterface->isTouchSequence());
+
+    // try cancel
+    m_seatInterface->setFocusedTouchSurface(serverSurface, QPointF(15, 26));
+    m_seatInterface->setTimestamp(7);
+    QCOMPARE(m_seatInterface->touchDown(QPointF(15, 26)), 0);
+    m_seatInterface->touchFrame();
+    m_seatInterface->cancelTouchSequence();
+    QVERIFY(sequenceCanceledSpy.wait());
+    QCOMPARE(sequenceStartedSpy.count(), 2);
+    QCOMPARE(sequenceEndedSpy.count(), 1);
+    QCOMPARE(sequenceCanceledSpy.count(), 1);
+    QCOMPARE(frameEndedSpy.count(), 7);
+    QCOMPARE(pointAddedSpy.count(), 2);
+    QCOMPARE(pointMovedSpy.count(), 1);
+    QCOMPARE(pointRemovedSpy.count(), 3);
+    QCOMPARE(touch->sequence().first()->position(), QPointF(0, 0));
+}
+
 QTEST_GUILESS_MAIN(TestWaylandSeat)
 #include "test_wayland_seat.moc"
