@@ -18,15 +18,25 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "buffer_interface.h"
+#include "display.h"
 #include "surface_interface.h"
 // Wayland
 #include <wayland-server.h>
+// EGL
+#include <EGL/egl.h>
+#include <QtGui/qopengl.h>
 
 namespace KWayland
 {
 namespace Server
 {
 
+namespace EGL
+{
+typedef GLboolean(*eglQueryWaylandBufferWL_func)(EGLDisplay dpy, struct wl_resource *buffer, EGLint attribute, EGLint *value);
+eglQueryWaylandBufferWL_func eglQueryWaylandBufferWL = nullptr;
+}
+
 class BufferInterface::Private
 {
 public:
@@ -38,6 +48,9 @@ public:
     wl_shm_buffer *shmBuffer;
     SurfaceInterface *surface;
     int refCount;
+    QSize size;
+
+    static BufferInterface *get(wl_resource *r);
 
 private:
     static void destroyListenerCallback(wl_listener *listener, void *data);
@@ -64,6 +77,15 @@ BufferInterface::Private *BufferInterface::Private::cast(wl_resource *r)
     return *it;
 }
 
+BufferInterface *BufferInterface::Private::get(wl_resource *r)
+{
+    Private *p = cast(r);
+    if (!p) {
+        return nullptr;
+    }
+    return p->q;
+}
+
 void BufferInterface::Private::imageBufferCleanupHandler(void *info)
 {
     Private *p = reinterpret_cast<Private*>(info);
@@ -86,6 +108,26 @@ BufferInterface::Private::Private(BufferInterface *q, wl_resource *resource, Sur
     s_buffers << this;
     listener.notify = destroyListenerCallback;
     wl_resource_add_destroy_listener(resource, &listener);
+    if (shmBuffer) {
+        size = QSize(wl_shm_buffer_get_width(shmBuffer), wl_shm_buffer_get_height(shmBuffer));
+    } else if (parent) {
+        EGLDisplay eglDisplay = parent->global()->display()->eglDisplay();
+        static bool resolved = false;
+        using namespace EGL;
+        if (!resolved && eglDisplay != EGL_NO_DISPLAY) {
+            eglQueryWaylandBufferWL = (eglQueryWaylandBufferWL_func)eglGetProcAddress("eglQueryWaylandBufferWL");
+            resolved = true;
+        }
+        if (eglQueryWaylandBufferWL) {
+            EGLint width, height;
+            bool valid = false;
+            valid = eglQueryWaylandBufferWL(eglDisplay, buffer, EGL_WIDTH, &width);
+            valid = valid && eglQueryWaylandBufferWL(eglDisplay, buffer, EGL_HEIGHT, &height);
+            if (valid) {
+                size = QSize(width, height);
+            }
+        }
+    }
 }
 
 BufferInterface::Private::~Private()
@@ -94,8 +136,21 @@ BufferInterface::Private::~Private()
     s_buffers.removeAll(this);
 }
 
+BufferInterface *BufferInterface::get(wl_resource *r)
+{
+    if (!r) {
+        return nullptr;
+    }
+    // TODO: verify it's a buffer
+    BufferInterface *b = Private::get(r);
+    if (b) {
+        return b;
+    }
+    return new BufferInterface(r, nullptr);
+}
+
 BufferInterface::BufferInterface(wl_resource *resource, SurfaceInterface *parent)
-    : QObject(parent)
+    : QObject()
     , d(new Private(this, resource, parent))
 {
 }
@@ -164,8 +219,8 @@ QImage BufferInterface::Private::createImage()
     s_accessCounter++;
     wl_shm_buffer_begin_access(shmBuffer);
     return std::move(QImage((const uchar*)wl_shm_buffer_get_data(shmBuffer),
-                            wl_shm_buffer_get_width(shmBuffer),
-                            wl_shm_buffer_get_height(shmBuffer),
+                            size.width(),
+                            size.height(),
                             wl_shm_buffer_get_stride(shmBuffer),
                             imageFormat,
                             &imageBufferCleanupHandler, this));
@@ -186,5 +241,24 @@ wl_shm_buffer *BufferInterface::shmBuffer()
     return d->shmBuffer;
 }
 
+wl_resource *BufferInterface::resource() const
+{
+    return d->buffer;
+}
+
+QSize BufferInterface::size() const
+{
+    return d->size;
+}
+
+void BufferInterface::setSize(const QSize &size)
+{
+    if (d->shmBuffer || d->size == size) {
+        return;
+    }
+    d->size = size;
+    emit sizeChanged();
+}
+
 }
 }
