@@ -20,6 +20,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "display.h"
 #include "compositor_interface.h"
 #include "datadevicemanager_interface.h"
+#include "logging_p.h"
 #include "output_interface.h"
 #include "seat_interface.h"
 #include "shell_interface.h"
@@ -29,9 +30,12 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include <QDebug>
 #include <QAbstractEventDispatcher>
 #include <QSocketNotifier>
+#include <QThread>
 
 #include <wayland-server.h>
 
+#include <EGL/egl.h>
+
 namespace KWayland
 {
 namespace Server
@@ -51,6 +55,7 @@ public:
     bool running = false;
     QList<OutputInterface*> outputs;
     QVector<ClientConnection*> clients;
+    EGLDisplay eglDisplay = EGL_NO_DISPLAY;
 
 private:
     Display *q;
@@ -63,17 +68,17 @@ Display::Private::Private(Display *q)
 
 void Display::Private::installSocketNotifier()
 {
-    if (!QCoreApplication::instance()) {
+    if (!QThread::currentThread()) {
         return;
     }
     int fd = wl_event_loop_get_fd(loop);
     if (fd == -1) {
-        qWarning() << "Did not get the file descriptor for the event loop";
+        qCWarning(KWAYLAND_SERVER) << "Did not get the file descriptor for the event loop";
         return;
     }
     QSocketNotifier *m_notifier = new QSocketNotifier(fd, QSocketNotifier::Read, q);
     QObject::connect(m_notifier, &QSocketNotifier::activated, q, [this] { flush(); } );
-    QObject::connect(QCoreApplication::eventDispatcher(), &QAbstractEventDispatcher::aboutToBlock, q, [this] { flush(); });
+    QObject::connect(QThread::currentThread()->eventDispatcher(), &QAbstractEventDispatcher::aboutToBlock, q, [this] { flush(); });
     setRunning(true);
 }
 
@@ -94,7 +99,7 @@ void Display::Private::flush()
         return;
     }
     if (wl_event_loop_dispatch(loop, 0) != 0) {
-        qWarning() << "Error on dispatching Wayland event loop";
+        qCWarning(KWAYLAND_SERVER) << "Error on dispatching Wayland event loop";
     }
     wl_display_flush_clients(display);
 }
@@ -209,7 +214,7 @@ DataDeviceManagerInterface *Display::createDataDeviceManager(QObject *parent)
 
 void Display::createShm()
 {
-    Q_ASSERT(d->running);
+    Q_ASSERT(d->display);
     wl_display_init_shm(d->display);
 }
 
@@ -292,5 +297,19 @@ ClientConnection *Display::createClient(int fd)
     return getConnection(c);
 }
 
+void Display::setEglDisplay(void *display)
+{
+    if (d->eglDisplay != EGL_NO_DISPLAY) {
+        qCWarning(KWAYLAND_SERVER) << "EGLDisplay cannot be changed";
+        return;
+    }
+    d->eglDisplay = (EGLDisplay)display;
+}
+
+void *Display::eglDisplay() const
+{
+    return d->eglDisplay;
+}
+
 }
 }
